package com.ywt.mg.core.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

/**
 * @author daiyihua
 * @create 2020-11-24 11:15
 * @program bank_service
 * @description Zip文件处理工具类
 **/
public class ZipFileUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(ZipFileUtil.class);
    /**
     * 功能：把 sourceDir 目录下的所有文件进行 zip 格式的压缩，保存为指定 zip 文件
     *
     * @param sourceDir 原文件
     * @param zipFile   压缩之后的文件
     */
    public static void zip(String sourceDir, String zipFile) {
        OutputStream os = null;
        ZipOutputStream zos = null;
        try {
            os = new FileOutputStream(zipFile);
            BufferedOutputStream bos = new BufferedOutputStream(os);
            zos = new ZipOutputStream(bos);
            File file = new File(sourceDir);
            String basePath = null;
            if (file.isDirectory()) {
                basePath = file.getPath();
            } else {//直接压缩单个文件时，取父目录
                basePath = file.getParent();
            }
            zipFile(file, basePath, zos);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (zos != null) {
                try {
                    zos.closeEntry();
                    zos.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    /**
     * 功能：执行文件压缩成zip文件
     *
     * @param source   原文件
     * @param basePath 待压缩文件根目录
     * @param zos      {@link ZipOutputStream}
     */
    private static void zipFile(File source, String basePath, ZipOutputStream zos) {
        File[] files = new File[0];
        if (source.isDirectory()) {
            files = source.listFiles();
        } else {
            files = new File[1];
            files[0] = source;
        }
        String pathName;//存相对路径(相对于待压缩的根目录)
        byte[] buf = new byte[1024];
        int length = 0;
        try {
            for (File file : files) {
                if (file.isDirectory()) {
                    pathName = file.getPath().substring(basePath.length() + 1) + "/";
                    zos.putNextEntry(new ZipEntry(pathName));
                    zipFile(file, basePath, zos);
                } else {
                    pathName = file.getPath().substring(basePath.length() + 1);
                    InputStream is = new FileInputStream(file);
                    BufferedInputStream bis = new BufferedInputStream(is);
                    zos.putNextEntry(new ZipEntry(pathName));
                    while ((length = bis.read(buf)) > 0) {
                        zos.write(buf, 0, length);
                    }
                    is.close();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 功能：解压 zip 文件，只能解压 zip 文件
     *
     * @param zipfile 压缩文件
     * @param destDir 解压路径
     */
    public static String unZip(String zipfile, String destDir) {
        byte b[] = new byte[1024];
        int length;
        ZipFile zipFile;
        // 获取第一个文件的名称
        String firstFileName = "";
        OutputStream outputStream = null;
        InputStream inputStream = null;
        try {
            zipFile = new ZipFile(new File(zipfile));
            Enumeration enumeration = zipFile.entries();
            ZipEntry zipEntry = null;
            int i = 0;
            while (enumeration.hasMoreElements()) {
                zipEntry = (ZipEntry) enumeration.nextElement();
                // 获取第一个文件的文件名
                if (i == 0) {
                    firstFileName = zipEntry.getName();
                }
                i++;
                File loadFile = new File(destDir + zipEntry.getName());
                // 判断是否为文件夹
                if (zipEntry.isDirectory()) {
                    loadFile.mkdirs();
                } else {
                    // 如果父类文件夹不存在，那就生成
                    if (!loadFile.getParentFile().exists()) {
                        loadFile.getParentFile().mkdirs();
                    }
                    outputStream = new FileOutputStream(loadFile);
                    inputStream = zipFile.getInputStream(zipEntry);
                    while ((length = inputStream.read(b)) > 0) {
                        outputStream.write(b, 0, length);
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
        LOGGER.info(firstFileName);
        return firstFileName;
    }

    /**
     * 压缩成ZIP  一次性压缩多个文件
     *
     * @param srcFiles 需要压缩的文件列表
     * @param zipFileName 压缩文件输出
     * @throws RuntimeException 压缩失败会抛出运行时异常
     */
    public static void stringFileUrlToZip(String zipFileName, List<String> srcFiles) throws Exception {
        long start = System.currentTimeMillis();
        List<File> fileList = new ArrayList<>();
        try {
            for (String srcFile : srcFiles) {
                File file = new File(srcFile);
                if (!Checker.isNone(file)){
                    fileList.add(file);
                }
            }
            toZip(zipFileName, fileList);
            long end = System.currentTimeMillis();
            LOGGER.info("ZipFileUtil#stringFileUrlToZip 压缩完成，耗时：" + (end - start) + " 毫秒");
        } catch (Exception e) {
            throw new RuntimeException("stringFileUrlToZip zip error from ZipUtils", e);
        }
    }
    /**
     * 压缩成ZIP  一次性压缩多个文件
     *
     * @param srcFiles 需要压缩的文件列表
     * @param zipFileName 压缩文件输出
     * @throws RuntimeException 压缩失败会抛出运行时异常
     */
    public static void toZip(String zipFileName, List<File> srcFiles) throws Exception {
        long start = System.currentTimeMillis();
        ZipOutputStream zos = null;
        try {
            FileOutputStream fileOutputStream = new FileOutputStream(zipFileName);
            zos = new ZipOutputStream(fileOutputStream);
            for (File srcFile : srcFiles) {
                compress(srcFile, zos, srcFile.getName(), true);
            }
            long end = System.currentTimeMillis();
            LOGGER.info("ZipFileUtil#toZip 压缩完成，耗时：" + (end - start) + " 毫秒");
        } catch (Exception e) {
            throw new RuntimeException("zip error from ZipUtils", e);
        } finally {
            if (zos != null) {
                try {
                    zos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 递归压缩方法
     *
     * @param sourceFile       源文件
     * @param zos              zip输出流
     * @param name             压缩后的名称
     * @param KeepDirStructure 是否保留原来的目录结构,true:保留目录结构;
     *                         false:所有文件跑到压缩包根目录下(注意：不保留目录结构可能会出现同名文件,会压缩失败)
     * @throws Exception
     */
    public static void compress(File sourceFile, ZipOutputStream zos, String name,
                                boolean KeepDirStructure) throws Exception {
        byte[] buf = new byte[1024];
        if (sourceFile.isFile()) {
            // 向zip输出流中添加一个zip实体，构造器中name为zip实体的文件的名字
            zos.putNextEntry(new ZipEntry(name));
            // copy文件到zip输出流中
            int len;
            FileInputStream in = new FileInputStream(sourceFile);
            while ((len = in.read(buf)) != -1) {
                zos.write(buf, 0, len);
            }
            // Complete the entry
            zos.closeEntry();
            in.close();
        } else {
            File[] listFiles = sourceFile.listFiles();
            if (listFiles == null || listFiles.length == 0) {
                // 需要保留原来的文件结构时,需要对空文件夹进行处理
                if (KeepDirStructure) {
                    // 空文件夹的处理
                    zos.putNextEntry(new ZipEntry(name + "/"));
                    // 没有文件，不需要文件的copy
                    zos.closeEntry();
                }
            } else {
                for (File file : listFiles) {
                    // 判断是否需要保留原来的文件结构
                    if (KeepDirStructure) {
                        // 注意：file.getName()前面需要带上父文件夹的名字加一斜杠,
                        // 不然最后压缩包中就不能保留原来的文件结构,即：所有文件都跑到压缩包根目录下了
                        compress(file, zos, name + "/" + file.getName(), KeepDirStructure);
                    } else {
                        compress(file, zos, file.getName(), KeepDirStructure);
                    }

                }
            }
        }
    }


}
